@@ -18,6 +18,7 @@ import logging
 
 from django.core.urlresolvers import reverse_lazy
 from django.utils.translation import ugettext_lazy as _
+
 from horizon import exceptions
 from horizon import forms
 from horizon import messages
@@ -44,33 +45,6 @@ class CommonData(forms.SelfHandlingForm):
             self.fields = collections.OrderedDict(
                 (k, self.fields[k]) for k in self.fields_order)
 
-    def clean(self):
-        cleaned_data = super(CommonData, self).clean()
-        name = cleaned_data.get('name')
-        try:
-            if self.request.user.is_superuser:
-                for attribute in bgpvpn_common.RT_FORMAT_ATTRIBUTES:
-                    if not cleaned_data.get(attribute):
-                        del cleaned_data[attribute]
-                    else:
-                        cleaned_data[attribute] = bgpvpn_common.format_rt(
-                            cleaned_data[attribute])
-                # if an admin user use the bgpvpn panel project
-                # tenant_id field doesn't exist
-                if not cleaned_data.get('tenant_id'):
-                    tenant_id = self.request.user.tenant_id
-                else:
-                    tenant_id = cleaned_data.get('tenant_id')
-                bgpvpn_api.bgpvpns_list(
-                    self.request, name=name, tenant_id=tenant_id)
-            else:
-                bgpvpn_api.bgpvpns_list(self.request, name=name)
-        except Exception:
-            msg = _('Unable to get BGPVPN with name %s') % name
-            exceptions.check_message(["Connection", "refused"], msg)
-            raise
-        return cleaned_data
-
     @staticmethod
     def _del_attributes(attributes, data):
         for attribute in attributes:
@@ -78,35 +52,39 @@ class CommonData(forms.SelfHandlingForm):
 
     def handle(self, request, data):
         params = {}
+        for key in bgpvpn_common.RT_FORMAT_ATTRIBUTES:
+            if key in data:
+                params[key] = bgpvpn_common.format_rt(data.pop(key, None))
         params.update(data)
+        error_msg = _('Something went wrong with BGPVPN %s') % data['name']
         try:
             if self.action == 'update':
+                error_msg = _('Failed to update BGPVPN %s') % data['name']
+                # attribute tenant_id is required in request when admin user is
+                # logged and bgpvpn form from admin menu is used
                 if request.user.is_superuser and data.get('tenant_id'):
                     attributes = ('bgpvpn_id', 'type', 'tenant_id')
+                # attribute tenant_id does not exist in request
+                # when non-admin user is logged
                 else:
                     attributes = ('bgpvpn_id', 'type')
                 self._del_attributes(attributes, params)
                 bgpvpn = bgpvpn_api.bgpvpn_update(request,
                                                   data['bgpvpn_id'],
                                                   **params)
-                success_action = 'modified'
+                msg = _('BGPVPN %s was successfully updated.') % data['name']
             elif self.action == 'create':
-                success_action = 'created'
+                error_msg = _('Failed to create BGPVPN %s') % data['name']
                 bgpvpn = bgpvpn_api.bgpvpn_create(request, **params)
+                msg = _('BGPVPN %s was successfully created.') % data['name']
             else:
                 raise Exception(
-                    _('Action type %s is not supported') % self.action)
-            msg = _('BGPVPN {name} was successfully {action}.').format(
-                name=data['name'],
-                action=success_action)
+                    _('Unsupported action type: %s') % self.action)
             LOG.debug(msg)
             messages.success(request, msg)
             return bgpvpn
         except Exception:
-            msg = _('Failed to {action} BGPVPN {name}').format(
-                action=self.action,
-                name=data['name'])
-            exceptions.handle(request, msg, redirect=self.failure_url)
+            exceptions.handle(request, error_msg, redirect=self.failure_url)
             return False
 
 
