@@ -98,15 +98,15 @@ _config_generator_config_files = [
     'opencontrail-driver.conf',
 ]
 
-def _get_config_generator_config_definition(config_file):
-    config_file_path = '../../etc/oslo-config-generator/%s' % conf
+def _get_config_generator_config_definition(conf_file):
+    config_file_path = '../../etc/oslo-config-generator/%s' % conf_file
     # oslo_config.sphinxconfiggen appends '.conf.sample' to the filename,
     # strip file extentension (.conf or .ini).
-    output_file_path = '_static/config-samples/%s' % conf.rsplit('.', 1)[0]
+    output_file_path = '_static/config-samples/%s' % conf_file.rsplit('.', 1)[0]
     return (config_file_path, output_file_path)
 
 
 config_generator_config_file = [
-    _get_config_generator_config_definition(conf)
-    for conf in _config_generator_config_files
+    _get_config_generator_config_definition(conf_file)
+    for conf_file in _config_generator_config_files
 ]
