@@ -14,6 +14,12 @@
 #    under the License.
 
 
+from neutron_lib.api.definitions import bgpvpn as bgpvpn_def
+from neutron_lib.api.definitions import bgpvpn_routes_control as bgpvpn_rc_def
+from neutron_lib.api.definitions import bgpvpn_vni as bgpvpn_vni_def
+from neutron_lib.plugins import directory
+
+
 def rtrd_list2str(list):
     """Format Route Target list to string"""
     if not list:
@@ -59,6 +65,10 @@ def filter_fields(resource, fields):
     return resource
 
 
+def is_extension_supported(plugin, ext_alias):
+    return ext_alias in plugin.supported_extension_aliases
+
+
 def make_bgpvpn_dict(bgpvpn, fields=None):
     res = {
         'id': bgpvpn['id'],
@@ -71,7 +81,14 @@ def make_bgpvpn_dict(bgpvpn, fields=None):
         'route_distinguishers': rtrd_str2list(bgpvpn['route_distinguishers']),
         'networks': bgpvpn.get('networks', []),
         'routers': bgpvpn.get('routers', []),
+        'ports': bgpvpn.get('ports', []),
     }
+    plugin = directory.get_plugin(bgpvpn_def.LABEL)
+    if is_extension_supported(plugin, bgpvpn_vni_def.ALIAS):
+        res[bgpvpn_vni_def.VNI] = bgpvpn.get(bgpvpn_vni_def.VNI)
+    if is_extension_supported(plugin, bgpvpn_rc_def.ALIAS):
+        res[bgpvpn_rc_def.LOCAL_PREF_KEY] = bgpvpn.get(
+            bgpvpn_rc_def.LOCAL_PREF_KEY)
     return filter_fields(res, fields)
 
 
@@ -91,13 +108,21 @@ def make_router_assoc_dict(id, tenant_id, bgpvpn_id, router_id, fields=None):
     return filter_fields(res, fields)
 
 
+def make_port_assoc_dict(id, tenant_id, bgpvpn_id, port_id, fields=None):
+    # NOTE(tmorin): fields need to be added here, this isn't used yet
+    res = {'id': id,
+           'tenant_id': tenant_id,
+           'bgpvpn_id': bgpvpn_id,
+           'port_id': port_id}
+    return filter_fields(res, fields)
+
+
 def get_bgpvpn_differences(current_dict, old_dict):
     """Compare 2 BGP VPN
 
-    - added elements (route_targets, import_targets or export_targets)
-    - removed elements (route_targets, import_targets or export_targets)
-    - changed values for keys in both dictionaries  (network_id,
-      route_targets, import_targets or export_targets)
+    - added keys
+    - removed keys
+    - changed values for keys in both dictionaries
     """
     set_current = set(current_dict.keys())
     set_old = set(old_dict.keys())
