@@ -16,7 +16,7 @@
 from sqlalchemy import orm
 from sqlalchemy import sql
 
-from neutron.db.models import external_net
+from neutron.db import api as db_api
 from neutron.db.models import l3
 from neutron.db import models_v2
 from neutron.debug import debug_agent
@@ -26,7 +26,6 @@ from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
 from neutron_lib.callbacks import resources
 from neutron_lib import constants as const
-from neutron_lib import exceptions as n_exc
 
 from oslo_log import helpers as log_helpers
 from oslo_log import log as logging
@@ -34,40 +33,32 @@ from oslo_log import log as logging
 from networking_bagpipe.agent.bgpvpn import rpc_client
 
 from networking_bgpvpn.neutron.db import bgpvpn_db
-from networking_bgpvpn.neutron.extensions import bgpvpn as bgpvpn_ext
-from networking_bgpvpn.neutron.services.common import constants
 from networking_bgpvpn.neutron.services.common import utils
-from networking_bgpvpn.neutron.services.service_drivers import driver_api
+from networking_bgpvpn.neutron.services.service_drivers.bagpipe \
+    import bagpipe_v2 as v2
+
 
 LOG = logging.getLogger(__name__)
 
 
-BAGPIPE_DRIVER_NAME = "bagpipe"
-
-
-class BGPVPNExternalNetAssociation(n_exc.NeutronException):
-    message = _("driver does not support associating an external"
-                "network to a BGPVPN")
-
-
 @log_helpers.log_method_call
+@db_api.context_manager.reader
 def get_network_info_for_port(context, port_id, network_id):
     """Get MAC, IP and Gateway IP addresses informations for a specific port"""
     try:
-        with context.session.begin(subtransactions=True):
-            net_info = (context.session.
-                        query(models_v2.Port.mac_address,
-                              models_v2.IPAllocation.ip_address,
-                              models_v2.Subnet.cidr,
-                              models_v2.Subnet.gateway_ip).
-                        join(models_v2.IPAllocation).
-                        join(models_v2.Subnet,
-                             models_v2.IPAllocation.subnet_id ==
-                             models_v2.Subnet.id).
-                        filter(models_v2.Subnet.ip_version == 4).
-                        filter(models_v2.Port.id == port_id).one())
+        net_info = (context.session.
+                    query(models_v2.Port.mac_address,
+                          models_v2.IPAllocation.ip_address,
+                          models_v2.Subnet.cidr,
+                          models_v2.Subnet.gateway_ip).
+                    join(models_v2.IPAllocation).
+                    join(models_v2.Subnet,
+                         models_v2.IPAllocation.subnet_id ==
+                         models_v2.Subnet.id).
+                    filter(models_v2.Subnet.ip_version == 4).
+                    filter(models_v2.Port.id == port_id).one())
 
-            (mac_address, ip_address, cidr, gateway_ip) = net_info
+        (mac_address, ip_address, cidr, gateway_ip) = net_info
     except orm.exc.NoResultFound:
         return
 
@@ -88,22 +79,23 @@ def get_network_info_for_port(context, port_id, network_id):
             'gateway_mac': gateway_mac[0] if gateway_mac else None}
 
 
+@db_api.context_manager.reader
 def get_gateway_mac(context, network_id):
-    with context.session.begin(subtransactions=True):
-        gateway_mac = (
-            context.session.
-            query(models_v2.Port.mac_address).
-            filter(
-                models_v2.Port.network_id == network_id,
-                (models_v2.Port.device_owner ==
-                 const.DEVICE_OWNER_ROUTER_INTF)
-            ).
-            one_or_none()
-        )
+    gateway_mac = (
+        context.session.
+        query(models_v2.Port.mac_address).
+        filter(
+            models_v2.Port.network_id == network_id,
+            (models_v2.Port.device_owner ==
+             const.DEVICE_OWNER_ROUTER_INTF)
+        ).
+        one_or_none()
+    )
 
-        return gateway_mac[0] if gateway_mac else None
+    return gateway_mac[0] if gateway_mac else None
 
 
+@db_api.context_manager.reader
 def get_network_ports(context, network_id):
     # NOTE(tmorin): currents callers don't look at detailed results
     # but only test if at least one result exist => can be optimized
@@ -113,6 +105,7 @@ def get_network_ports(context, network_id):
                    models_v2.Port.admin_state_up == sql.true()).all())
 
 
+@db_api.context_manager.reader
 def get_router_ports(context, router_id):
     return (
         context.session.query(models_v2.Port).
@@ -123,6 +116,7 @@ def get_router_ports(context, router_id):
     )
 
 
+@db_api.context_manager.reader
 def get_router_bgpvpn_assocs(context, router_id):
     return (
         context.session.query(bgpvpn_db.BGPVPNRouterAssociation).
@@ -132,6 +126,7 @@ def get_router_bgpvpn_assocs(context, router_id):
     )
 
 
+@db_api.context_manager.reader
 def get_network_bgpvpn_assocs(context, net_id):
     return (
         context.session.query(bgpvpn_db.BGPVPNNetAssociation).
@@ -141,6 +136,7 @@ def get_network_bgpvpn_assocs(context, net_id):
     )
 
 
+@db_api.context_manager.reader
 def get_bgpvpns_of_router_assocs_by_network(context, net_id):
     return (
         context.session.query(bgpvpn_db.BGPVPN).
@@ -154,13 +150,13 @@ def get_bgpvpns_of_router_assocs_by_network(context, net_id):
     )
 
 
-def network_is_external(context, net_id):
-    try:
-        context.session.query(external_net.ExternalNetwork).filter_by(
-            network_id=net_id).one()
-        return True
-    except orm.exc.NoResultFound:
-        return False
+@db_api.context_manager.reader
+def get_networks_for_router(context, router_id):
+    ports = get_router_ports(context, router_id)
+    if ports:
+        return set([port['network_id'] for port in ports])
+    else:
+        return []
 
 
 def _log_callback_processing_exception(resource, event, trigger, kwargs, e):
@@ -174,7 +170,7 @@ def _log_callback_processing_exception(resource, event, trigger, kwargs, e):
                    'exc': e})
 
 
-class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
+class BaGPipeBGPVPNDriver(v2.BaGPipeBGPVPNDriver):
 
     """BGPVPN Service Driver class for BaGPipe"""
 
@@ -191,10 +187,6 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
                            resources.PORT,
                            events.AFTER_DELETE)
 
-        registry.subscribe(self.registry_router_interface_created,
-                           resources.ROUTER_INTERFACE,
-                           events.AFTER_CREATE)
-
         registry.subscribe(self.registry_router_interface_deleted,
                            resources.ROUTER_INTERFACE,
                            events.AFTER_DELETE)
@@ -216,19 +208,32 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
     def _format_bgpvpn_network_route_targets(self, bgpvpns):
         """Format BGPVPN network informations (VPN type and route targets)
 
-        {
+        [{
             'type': 'l3',
             'route_targets': ['12345:1', '12345:2'],
             'import_targets': ['12345:3'],
             'export_targets': ['12345:4']
+        },
+        {
+            'type': 'l3',
+            'route_targets': ['12346:1']
+        },
+        {
+            'type': 'l2',
+            'route_targets': ['12347:1']
         }
+        ]
 
         to
 
         {
             'l3vpn' : {
-                'import_rt': ['12345:1', '12345:2', '12345:3'],
-                'export_rt': ['12345:1', '12345:2', '12345:4']
+                'import_rt': ['12345:1', '12345:2', '12345:3', '12346:1'],
+                'export_rt': ['12345:1', '12345:2', '12345:4', '12346:1']
+            },
+            'l2vpn' : {
+                'import_rt': ['12347:1'],
+                'export_rt': ['12347:1']
             }
         }
 
@@ -260,10 +265,10 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
                     bgpvpn['export_targets']
                 )
 
-        # FIXME: Only returning l3vpn route targets. Must be modified to also
-        # return l2vpn ones in the future
-        if 'l2vpn' in bgpvpn_rts:
-            bgpvpn_rts.pop('l2vpn')
+            for attribute in ('import_rt', 'export_rt'):
+                if bgpvpn_rts[bgpvpn['type'] + 'vpn'][attribute]:
+                    bgpvpn_rts[bgpvpn['type'] + 'vpn'][attribute] = list(
+                        set(bgpvpn_rts[bgpvpn['type'] + 'vpn'][attribute]))
 
         return bgpvpn_rts
 
@@ -278,6 +283,13 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
                                                                    network_id)
         )
 
+    def _networks_for_bgpvpn(self, context, bgpvpn):
+        networks = []
+        networks.extend(bgpvpn['networks'])
+        for router_id in bgpvpn['routers']:
+            networks.extend(get_networks_for_router(context, router_id))
+        return list(set(networks))
+
     def _retrieve_bgpvpn_network_info_for_port(self, context, port):
         """Retrieve BGP VPN network informations for a specific port
 
@@ -325,66 +337,50 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
 
         return bgpvpn_network_info
 
+    @db_api.context_manager.reader
     def retrieve_bgpvpns_of_router_assocs_by_network(self, context,
                                                      network_id):
         return [self.bgpvpn_db._make_bgpvpn_dict(bgpvpn) for bgpvpn in
                 get_bgpvpns_of_router_assocs_by_network(context, network_id)]
 
-    def _common_precommit_checks(self, bgpvpn):
-        # No support yet for specifying route distinguishers
-        if bgpvpn.get('route_distinguishers', None):
-            raise bgpvpn_ext.BGPVPNRDNotSupported(driver=BAGPIPE_DRIVER_NAME)
-
-    def create_bgpvpn_precommit(self, context, bgpvpn):
-        # Only l3 type is supported
-        if bgpvpn['type'] != constants.BGPVPN_L3:
-            raise bgpvpn_ext.BGPVPNTypeNotSupported(driver=BAGPIPE_DRIVER_NAME,
-                                                    type=bgpvpn['type'])
-
-        self._common_precommit_checks(bgpvpn)
-
     def delete_bgpvpn_postcommit(self, context, bgpvpn):
-        for net_id in bgpvpn['networks']:
+        for net_id in self._networks_for_bgpvpn(context, bgpvpn):
             if get_network_ports(context, net_id):
                 # Format BGPVPN before sending notification
                 self.agent_rpc.delete_bgpvpn(
                     context,
                     self._format_bgpvpn(context, bgpvpn, net_id))
 
-    def update_bgpvpn_precommit(self, context, old_bgpvpn, bgpvpn):
-        self._common_precommit_checks(bgpvpn)
-
     def update_bgpvpn_postcommit(self, context, old_bgpvpn, bgpvpn):
+        super(BaGPipeBGPVPNDriver, self).update_bgpvpn_postcommit(
+            context, old_bgpvpn, bgpvpn)
+
         (added_keys, removed_keys, changed_keys) = (
             utils.get_bgpvpn_differences(bgpvpn, old_bgpvpn))
-        for net_id in bgpvpn['networks']:
-            if (get_network_ports(context, net_id)):
-                if ((key in added_keys for key in ('route_targets',
-                                                   'import_targets',
-                                                   'export_targets')) or
-                    (key in removed_keys for key in ('route_targets',
-                                                     'import_targets',
-                                                     'export_targets')) or
-                    (key in changed_keys for key in ('route_targets',
-                                                     'import_targets',
-                                                     'export_targets'))):
+        ATTRIBUTES_TO_IGNORE = set('name')
+        moving_keys = added_keys | removed_keys | changed_keys
+        if len(moving_keys ^ ATTRIBUTES_TO_IGNORE):
+            for net_id in self._networks_for_bgpvpn(context, bgpvpn):
+                if (get_network_ports(context, net_id)):
                     self._update_bgpvpn_for_network(context, net_id, bgpvpn)
 
-    def create_net_assoc_precommit(self, context, net_assoc):
-        if network_is_external(context, net_assoc['network_id']):
-            raise BGPVPNExternalNetAssociation()
-
-    def create_net_assoc_postcommit(self, context, net_assoc):
-        if get_network_ports(context, net_assoc['network_id']):
-            bgpvpn = self.get_bgpvpn(context, net_assoc['bgpvpn_id'])
-            self._update_bgpvpn_for_network(context,
-                                            net_assoc['network_id'], bgpvpn)
+    def _update_bgpvpn_for_net_with_id(self, context, network_id, bgpvpn_id):
+        if get_network_ports(context, network_id):
+            bgpvpn = self.get_bgpvpn(context, bgpvpn_id)
+            self._update_bgpvpn_for_network(context, network_id, bgpvpn)
 
     def _update_bgpvpn_for_network(self, context, net_id, bgpvpn):
         formated_bgpvpn = self._format_bgpvpn(context, bgpvpn, net_id)
         self.agent_rpc.update_bgpvpn(context,
                                      formated_bgpvpn)
 
+    def create_net_assoc_postcommit(self, context, net_assoc):
+        super(BaGPipeBGPVPNDriver, self).create_net_assoc_postcommit(context,
+                                                                     net_assoc)
+        self._update_bgpvpn_for_net_with_id(context,
+                                            net_assoc['network_id'],
+                                            net_assoc['bgpvpn_id'])
+
     def delete_net_assoc_postcommit(self, context, net_assoc):
         if get_network_ports(context, net_assoc['network_id']):
             bgpvpn = self.get_bgpvpn(context, net_assoc['bgpvpn_id'])
@@ -401,7 +397,7 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
                      port['id'])
             return True
 
-        if network_is_external(context, port['network_id']):
+        if v2.network_is_external(context, port['network_id']):
             LOG.info("Port %s is on an external network, we'll do nothing",
                      port['id'])
             return True
@@ -463,25 +459,26 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
                                                port[portbindings.HOST_ID])
 
     def create_router_assoc_postcommit(self, context, router_assoc):
-        ports = get_router_ports(context, router_assoc['router_id'])
-        if ports:
-            net_ids = set([port['network_id'] for port in ports])
-            for net_id in net_ids:
-                net_assoc = {'network_id': net_id,
-                             'bgpvpn_id': router_assoc['bgpvpn_id']}
-                self.create_net_assoc_postcommit(context, net_assoc)
+        super(BaGPipeBGPVPNDriver, self).create_router_assoc_postcommit(
+            context, router_assoc)
+        for net_id in get_networks_for_router(context,
+                                              router_assoc['router_id']):
+            self._update_bgpvpn_for_net_with_id(context,
+                                                net_id,
+                                                router_assoc['bgpvpn_id'])
 
     def delete_router_assoc_postcommit(self, context, router_assoc):
-        ports = get_router_ports(context, router_assoc['router_id'])
-        if ports:
-            net_ids = set([port['network_id'] for port in ports])
-            for net_id in net_ids:
-                net_assoc = {'network_id': net_id,
-                             'bgpvpn_id': router_assoc['bgpvpn_id']}
-                self.delete_net_assoc_postcommit(context, net_assoc)
+        for net_id in get_networks_for_router(context,
+                                              router_assoc['router_id']):
+            net_assoc = {'network_id': net_id,
+                         'bgpvpn_id': router_assoc['bgpvpn_id']}
+            self.delete_net_assoc_postcommit(context, net_assoc)
 
     @log_helpers.log_method_call
     def notify_router_interface_created(self, context, router_id, net_id):
+        super(BaGPipeBGPVPNDriver, self).notify_router_interface_created(
+            context, router_id, net_id)
+
         net_assocs = get_network_bgpvpn_assocs(context, net_id)
         router_assocs = get_router_bgpvpn_assocs(context, router_id)
 
@@ -494,12 +491,15 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
                 self._update_bgpvpn_for_network(context, net_id, bgpvpn)
 
         for router_assoc in router_assocs:
-            net_assoc = {'network_id': net_id,
-                         'bgpvpn_id': router_assoc['bgpvpn_id']}
-            self.create_net_assoc_postcommit(context, net_assoc)
+            self._update_bgpvpn_for_net_with_id(context,
+                                                net_id,
+                                                router_assoc['bgpvpn_id'])
 
     @log_helpers.log_method_call
     def notify_router_interface_deleted(self, context, router_id, net_id):
+        super(BaGPipeBGPVPNDriver, self).notify_router_interface_deleted(
+            context, router_id, net_id)
+
         net_assocs = get_network_bgpvpn_assocs(context, net_id)
         router_assocs = get_router_bgpvpn_assocs(context, router_id)
 
@@ -536,16 +536,10 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
                                                kwargs, e)
 
     @log_helpers.log_method_call
-    def registry_router_interface_created(self, resource, event, trigger,
-                                          **kwargs):
-        try:
-            context = kwargs['context']
-            router_id = kwargs['router_id']
-            net_id = kwargs['port']['network_id']
-            self.notify_router_interface_created(context, router_id, net_id)
-        except Exception as e:
-            _log_callback_processing_exception(resource, event, trigger,
-                                               kwargs, e)
+    def registry_router_interface_before_delete(self, resource, event, trigger,
+                                                **kwargs):
+        # we don't do anything for this event, contrarily to parent class
+        pass
 
     @log_helpers.log_method_call
     def registry_router_interface_deleted(self, resource, event, trigger,
