@@ -13,16 +13,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import json
 import uuid
 
+from debtcollector import removals
 from oslo_log import log
+from oslo_serialization import jsonutils
 from oslo_utils import uuidutils
 
 from neutron_lib import exceptions as n_exc
 
-from networking_bgpvpn._i18n import _LI
-
 from networking_bgpvpn.neutron.extensions import bgpvpn as bgpvpn_ext
 from networking_bgpvpn.neutron.services.common import constants
 from networking_bgpvpn.neutron.services.common import utils
@@ -36,7 +35,14 @@ OPENCONTRAIL_BGPVPN_DRIVER_NAME = 'OpenContrail'
 
 LOG = log.getLogger(__name__)
 
+MESSAGE = ("replaced by a new driver which could be found in the Contrail "
+           "Neutron monolithic core plugin tree: 'neutron_plugin_contrail."
+           "plugins.opencontrail.networking_bgpvpn.contrail."
+           "ContrailBGPVPNDriver'")
 
+
+@removals.removed_class('OpenContrailBGPVPNDriver', version='Queens',
+                        removal_version='Rocky', message=MESSAGE)
 class OpenContrailBGPVPNDriver(driver_api.BGPVPNDriverBase):
     """BGP VPN Service Driver class for OpenContrail."""
 
@@ -105,12 +111,14 @@ class OpenContrailBGPVPNDriver(driver_api.BGPVPNDriverBase):
         raise n_exc.NetworkNotFound(net_id=network_id)
 
     def _set_bgpvpn_association(self, oc_client, operation, bgpvpn,
-                                networks=[]):
+                                networks=None):
+        if networks is None:
+            networks = []
         for network_id in networks:
             try:
                 net_ri_id = self._get_ri_id_of_network(oc_client, network_id)
             except n_exc.NetworkNotFound:
-                LOG.info(_LI("Network %s not found, cleanup route targets"),
+                LOG.info("Network %s not found, cleanup route targets",
                          network_id)
                 rts_fq_name = (bgpvpn['route_targets'] +
                                bgpvpn['import_targets'] +
@@ -184,7 +192,7 @@ class OpenContrailBGPVPNDriver(driver_api.BGPVPNDriverBase):
         bgpvpns = []
         for kv_dict in oc_client.kv_store('RETRIEVE'):
             try:
-                value = json.loads(kv_dict['value'])
+                value = jsonutils.loads(kv_dict['value'])
             except ValueError:
                 continue
             if (isinstance(value, dict) and
@@ -202,7 +210,7 @@ class OpenContrailBGPVPNDriver(driver_api.BGPVPNDriverBase):
     def _clean_bgpvpn_assoc(self, oc_client, bgpvpn_id):
         for kv_dict in oc_client.kv_store('RETRIEVE'):
             try:
-                value = json.loads(kv_dict['value'])
+                value = jsonutils.loads(kv_dict['value'])
             except ValueError:
                 continue
             if (isinstance(value, dict) and
@@ -218,7 +226,7 @@ class OpenContrailBGPVPNDriver(driver_api.BGPVPNDriverBase):
         oc_client = self._get_opencontrail_api_client(context)
 
         try:
-            bgpvpn = json.loads(oc_client.kv_store('RETRIEVE', key=id))
+            bgpvpn = jsonutils.loads(oc_client.kv_store('RETRIEVE', key=id))
         except (oc_exc.OpenContrailAPINotFound, ValueError):
             raise bgpvpn_ext.BGPVPNNotFound(id=id)
 
@@ -307,7 +315,7 @@ class OpenContrailBGPVPNDriver(driver_api.BGPVPNDriverBase):
         oc_client = self._get_opencontrail_api_client(context)
 
         try:
-            net_assoc = json.loads(
+            net_assoc = jsonutils.loads(
                 oc_client.kv_store('RETRIEVE', key=assoc_id))
         except (oc_exc.OpenContrailAPINotFound, ValueError):
             raise bgpvpn_ext.BGPVPNNetAssocNotFound(id=assoc_id,
@@ -365,7 +373,7 @@ class OpenContrailBGPVPNDriver(driver_api.BGPVPNDriverBase):
         bgpvpn_net_assocs = []
         for kv_dict in oc_client.kv_store('RETRIEVE'):
             try:
-                value = json.loads(kv_dict['value'])
+                value = jsonutils.loads(kv_dict['value'])
             except ValueError:
                 continue
             if (isinstance(value, dict) and
