@@ -17,11 +17,14 @@ import copy
 import mock
 import webob.exc
 
+from oslo_config import cfg
+
+from neutron.api.rpc.handlers import resources_rpc
+from neutron.db import agents_db
+from neutron.db import db_base_plugin_v2
 from neutron.debug import debug_agent
-
-from neutron.plugins.ml2 import config as ml2_config
+from neutron.plugins.ml2 import plugin as ml2_plugin
 from neutron.plugins.ml2 import rpc as ml2_rpc
-
 from neutron.tests.common import helpers
 
 from neutron_lib.api.definitions import portbindings
@@ -32,18 +35,27 @@ from neutron_lib.plugins import directory
 from networking_bgpvpn.neutron.services.service_drivers.bagpipe import bagpipe
 from networking_bgpvpn.tests.unit.services import test_plugin
 
+from networking_bagpipe.objects import bgpvpn as objs
 
-def _expected_formatted_bgpvpn(id, net_id, rt, gateway_mac=None):
+
+def _expected_formatted_bgpvpn(id, net_id, rt=None, gateway_mac=None):
         return {'id': id,
                 'network_id': net_id,
-                'l3vpn': {'import_rt': rt,
-                          'export_rt': rt},
-                'gateway_mac': gateway_mac}
+                'l3vpn': {'import_rt': rt or mock.ANY,
+                          'export_rt': rt or mock.ANY},
+                'gateway_mac': gateway_mac or mock.ANY}
+
+
+class TestCorePluginWithAgents(db_base_plugin_v2.NeutronDbPluginV2,
+                               agents_db.AgentDbMixin):
+    pass
 
 
 class TestBagpipeCommon(test_plugin.BgpvpnTestCaseMixin):
 
-    def setUp(self, plugin=None):
+    def setUp(self, plugin=None,
+              driver=('networking_bgpvpn.neutron.services.service_drivers.'
+                      'bagpipe.bagpipe.BaGPipeBGPVPNDriver')):
         self.mocked_rpc = mock.patch(
             'networking_bagpipe.agent.bgpvpn.rpc_client'
             '.BGPVPNAgentNotifyApi').start().return_value
@@ -53,9 +65,13 @@ class TestBagpipeCommon(test_plugin.BgpvpnTestCaseMixin):
         self.mock_update_rpc = self.mocked_rpc.update_bgpvpn
         self.mock_delete_rpc = self.mocked_rpc.delete_bgpvpn
 
-        provider = ('networking_bgpvpn.neutron.services.service_drivers.'
-                    'bagpipe.bagpipe.BaGPipeBGPVPNDriver')
-        super(TestBagpipeCommon, self).setUp(service_provider=provider,
+        mock.patch(
+            'neutron.common.rpc.get_client').start().return_value
+
+        if not plugin:
+            plugin = '%s.%s' % (__name__, TestCorePluginWithAgents.__name__)
+
+        super(TestBagpipeCommon, self).setUp(service_provider=driver,
                                              core_plugin=plugin)
 
         self.ctxt = n_context.Context('fake_user', self._tenant_id)
@@ -71,6 +87,195 @@ class TestBagpipeCommon(test_plugin.BgpvpnTestCaseMixin):
                                                         {'network': n_dict})}
 
 
+class AnyOfClass(object):
+
+    def __init__(self, cls):
+        self._class = cls
+
+    def __eq__(self, other):
+        return isinstance(other, self._class)
+
+    def __repr__(self):
+        return "AnyOfClass<%s>" % self._class.__name__
+
+
+class TestBagpipeOVOPushPullMixin(object):
+    # tests for OVO-based push notifications go here
+
+    @mock.patch.object(resources_rpc.ResourcesPushRpcApi, 'push')
+    def test_bgpvpn_update_name_only(self, mocked_push):
+        with self.bgpvpn() as bgpvpn:
+            self._update('bgpvpn/bgpvpns',
+                         bgpvpn['bgpvpn']['id'],
+                         {'bgpvpn': {'name': 'newname'}})
+            # check that no RPC push is done for BGPVPN objects
+            self.assertTrue(
+                mocked_push.call_count == 0 or
+                (not any([isinstance(ovo, objs.BGPVPNNetAssociation)
+                          for ovo in mocked_push.mock_calls[0][1][1]]) and
+                 not any([isinstance(ovo, objs.BGPVPNRouterAssociation)
+                          for ovo in mocked_push.mock_calls[0][1][1]])
+                 )
+            )
+
+    @mock.patch.object(resources_rpc.ResourcesPushRpcApi, 'push')
+    def test_bgpvpn_update_rts_no_assoc(self, mocked_push):
+        with self.bgpvpn() as bgpvpn:
+            self._update('bgpvpn/bgpvpns',
+                         bgpvpn['bgpvpn']['id'],
+                         {'bgpvpn': {'route_targets': ['64512:43']}})
+            # check that no RPC push is done for BGPVPN objects
+            self.assertTrue(
+                mocked_push.call_count == 0 or
+                (not any([isinstance(ovo, objs.BGPVPNNetAssociation)
+                          for ovo in mocked_push.mock_calls[0][1][1]]) and
+                 not any([isinstance(ovo, objs.BGPVPNRouterAssociation)
+                          for ovo in mocked_push.mock_calls[0][1][1]])
+                 )
+            )
+
+    @mock.patch.object(resources_rpc.ResourcesPushRpcApi, '_push')
+    def test_bgpvpn_update_delete_rts_with_assocs(self, mocked_push):
+        with self.bgpvpn(do_delete=False) as bgpvpn, \
+                self.network() as net, \
+                self.router(tenant_id=self._tenant_id) as router, \
+                self.assoc_net(bgpvpn['bgpvpn']['id'],
+                               net['network']['id'],
+                               do_disassociate=False), \
+                self.assoc_router(bgpvpn['bgpvpn']['id'],
+                                  router['router']['id'],
+                                  do_disassociate=False):
+            mocked_push.reset_mock()
+            self._update('bgpvpn/bgpvpns',
+                         bgpvpn['bgpvpn']['id'],
+                         {'bgpvpn': {'route_targets': ['64512:43']}})
+
+            mocked_push.assert_any_call(mock.ANY, 'BGPVPNNetAssociation',
+                                        mock.ANY, 'updated')
+            mocked_push.assert_any_call(mock.ANY, 'BGPVPNRouterAssociation',
+                                        mock.ANY, 'updated')
+
+            mocked_push.reset_mock()
+
+            # delete BGPVPN
+            self._delete('bgpvpn/bgpvpns',
+                         bgpvpn['bgpvpn']['id'])
+
+            # after delete
+            mocked_push.assert_any_call(mock.ANY, 'BGPVPNNetAssociation',
+                                        mock.ANY, 'deleted')
+            mocked_push.assert_any_call(mock.ANY, 'BGPVPNRouterAssociation',
+                                        mock.ANY, 'deleted')
+
+    @mock.patch.object(resources_rpc.ResourcesPushRpcApi, 'push')
+    def test_net_assoc_create_delete(self, mocked_push):
+        with self.network() as net, \
+                self.bgpvpn() as bgpvpn:
+            mocked_push.reset_mock()
+            with self.assoc_net(bgpvpn['bgpvpn']['id'],
+                                net['network']['id']):
+
+                mocked_push.assert_called_once_with(mock.ANY, mock.ANY,
+                                                    'created')
+
+                ovos_in_call = mocked_push.mock_calls[0][1][1]
+
+                self.assertEqual(
+                    [AnyOfClass(objs.BGPVPNNetAssociation)],
+                    ovos_in_call
+                    )
+
+                mocked_push.reset_mock()
+
+            # after net assoc delete
+            mocked_push.assert_called_once_with(mock.ANY, mock.ANY, 'deleted')
+
+            ovos_in_call = mocked_push.mock_calls[0][1][1]
+
+            self.assertEqual(
+                [AnyOfClass(objs.BGPVPNNetAssociation)],
+                ovos_in_call
+                )
+
+    @mock.patch.object(resources_rpc.ResourcesPushRpcApi, 'push')
+    def test_router_assoc_create_delete(self, mocked_push):
+        with self.router(tenant_id=self._tenant_id) as router, \
+                self.bgpvpn() as bgpvpn:
+            mocked_push.reset_mock()
+            with self.assoc_router(bgpvpn['bgpvpn']['id'],
+                                   router['router']['id']):
+
+                mocked_push.assert_called_once_with(mock.ANY, mock.ANY,
+                                                    'created')
+
+                ovos_in_call = mocked_push.mock_calls[0][1][1]
+
+                self.assertEqual(
+                    [AnyOfClass(objs.BGPVPNRouterAssociation)],
+                    ovos_in_call
+                    )
+
+                mocked_push.reset_mock()
+
+            # after router assoc delete
+            mocked_push.assert_called_once_with(mock.ANY, mock.ANY, 'deleted')
+
+            ovos_in_call = mocked_push.mock_calls[0][1][1]
+
+            self.assertEqual(
+                [AnyOfClass(objs.BGPVPNRouterAssociation)],
+                ovos_in_call
+                )
+
+    @mock.patch.object(resources_rpc.ResourcesPushRpcApi, 'push')
+    def test_port_assoc_crud(self, mocked_push):
+        with self.port() as port, \
+                self.bgpvpn() as bgpvpn:
+            mocked_push.reset_mock()
+            with self.assoc_port(bgpvpn['bgpvpn']['id'],
+                                 port['port']['id']) as port_assoc:
+
+                mocked_push.assert_called_once_with(mock.ANY, mock.ANY,
+                                                    'created')
+
+                ovos_in_call = mocked_push.mock_calls[0][1][1]
+
+                self.assertEqual(
+                    [AnyOfClass(objs.BGPVPNPortAssociation)],
+                    ovos_in_call
+                    )
+
+                mocked_push.reset_mock()
+
+                self._update(
+                    ('bgpvpn/bgpvpns/%s/port_associations' %
+                     bgpvpn['bgpvpn']['id']),
+                    port_assoc['port_association']['id'],
+                    {'port_association': {'advertise_fixed_ips': False}})
+
+                mocked_push.assert_called_once_with(mock.ANY, mock.ANY,
+                                                    'updated')
+
+                ovos_in_call = mocked_push.mock_calls[0][1][1]
+
+                self.assertEqual(
+                    [AnyOfClass(objs.BGPVPNPortAssociation)],
+                    ovos_in_call
+                    )
+
+                mocked_push.reset_mock()
+
+            # after port assoc delete
+            mocked_push.assert_called_once_with(mock.ANY, mock.ANY, 'deleted')
+
+            ovos_in_call = mocked_push.mock_calls[0][1][1]
+
+            self.assertEqual(
+                [AnyOfClass(objs.BGPVPNPortAssociation)],
+                ovos_in_call
+                )
+
+
 class TestBagpipeServiceDriver(TestBagpipeCommon):
 
     def test_create_bgpvpn_l2_fails(self):
@@ -199,6 +404,26 @@ class TestBagpipeServiceDriver(TestBagpipeCommon):
                         mock.ANY,
                         _expected_formatted_bgpvpn(id, net_id, rt))
 
+    def test_bagpipe_update_bgpvpn_with_router_assoc(self):
+        with self.network() as net, \
+                self.subnet(network=net) as subnet, \
+                self.router(tenant_id=self._tenant_id) as router, \
+                self.bgpvpn() as bgpvpn, \
+                self.assoc_router(bgpvpn['bgpvpn']['id'],
+                                  router['router']['id']), \
+                self.port(subnet=subnet):
+            self._router_interface_action('add',
+                                          router['router']['id'],
+                                          subnet['subnet']['id'],
+                                          None)
+            update_data = {'bgpvpn': {'route_targets': ['6543:21']}}
+            self.mock_update_rpc.reset_mock()
+            self._update('bgpvpn/bgpvpns', bgpvpn['bgpvpn']['id'], update_data)
+            self.mock_update_rpc.assert_called_once_with(
+                mock.ANY,
+                _expected_formatted_bgpvpn(bgpvpn['bgpvpn']['id'],
+                                           net['network']['id']))
+
     def test_bagpipe_delete_bgpvpn(self):
         mocked_delete = self.mocked_rpc.delete_bgpvpn
         with self.port() as port1:
@@ -213,6 +438,26 @@ class TestBagpipeServiceDriver(TestBagpipeCommon):
                         mock.ANY,
                         _expected_formatted_bgpvpn(id, net_id, rt))
 
+    def test_bagpipe_delete_bgpvpn_with_router_assoc(self):
+        with self.network() as net, \
+                self.subnet(network=net) as subnet, \
+                self.router(tenant_id=self._tenant_id) as router, \
+                self.bgpvpn(do_delete=False) as bgpvpn, \
+                self.assoc_router(bgpvpn['bgpvpn']['id'],
+                                  router['router']['id'],
+                                  do_disassociate=False), \
+                self.port(subnet=subnet):
+            self._router_interface_action('add',
+                                          router['router']['id'],
+                                          subnet['subnet']['id'],
+                                          None)
+            self.mock_delete_rpc.reset_mock()
+            self._delete('bgpvpn/bgpvpns', bgpvpn['bgpvpn']['id'])
+            self.mocked_rpc.delete_bgpvpn.assert_called_once_with(
+                mock.ANY,
+                _expected_formatted_bgpvpn(bgpvpn['bgpvpn']['id'],
+                                           net['network']['id']))
+
     def test_bagpipe_callback_to_rpc_update_port_after_router_itf_added(self):
         driver = self.bgpvpn_plugin.driver
         with self.network() as net, \
@@ -280,17 +525,23 @@ BGPVPN_INFO = {'mac_address': 'de:ad:00:00:be:ef',
                }
 
 
-class TestBagpipeServiceDriverCallbacks(TestBagpipeCommon):
+class TestCorePluginML2WithAgents(ml2_plugin.Ml2Plugin,
+                                  agents_db.AgentDbMixin):
+    pass
+
+
+class TestBagpipeServiceDriverCallbacks(TestBagpipeCommon,
+                                        TestBagpipeOVOPushPullMixin):
+
     '''Check that receiving callbacks results in RPC calls to the agent'''
 
-    _plugin_name = 'neutron.plugins.ml2.plugin.Ml2Plugin'
-
     def setUp(self):
-        ml2_config.cfg.CONF.set_override('mechanism_drivers',
-                                         ['logger', 'fake_agent'],
-                                         'ml2')
+        cfg.CONF.set_override('mechanism_drivers',
+                              ['logger', 'fake_agent'],
+                              'ml2')
 
-        super(TestBagpipeServiceDriverCallbacks, self).setUp(self._plugin_name)
+        super(TestBagpipeServiceDriverCallbacks, self).setUp(
+            "%s.%s" % (__name__, TestCorePluginML2WithAgents.__name__))
 
         self.port_create_status = 'DOWN'
         self.plugin = directory.get_plugin()
@@ -737,3 +988,124 @@ class TestBagpipeServiceDriverCallbacks(TestBagpipeCommon):
             self.assertFalse(self.mock_attach_rpc.called)
             self.assertFalse(self.mock_detach_rpc.called)
             self.assertTrue(log_exc.called)
+
+    def test_format_bgpvpn_network_route_targets(self):
+        driver = self.bgpvpn_plugin.driver
+        bgpvpns = [{
+            'type': 'l3',
+            'route_targets': ['12345:1', '12345:2', '12345:3'],
+            'import_targets': ['12345:2', '12345:3'],
+            'export_targets': ['12345:3', '12345:4']
+        },
+            {
+                'type': 'l3',
+                'route_targets': ['12345:3', '12346:1']
+            },
+            {
+                'type': 'l2',
+                'route_targets': ['12347:1']
+            }]
+        result = driver._format_bgpvpn_network_route_targets(bgpvpns)
+        expected = {
+            'l3vpn': {
+                'import_rt': ['12345:1', '12345:2', '12345:3', '12346:1'],
+                'export_rt': ['12345:1', '12345:2', '12345:3', '12345:4',
+                              '12346:1']
+            },
+            'l2vpn': {
+                'import_rt': ['12347:1'],
+                'export_rt': ['12347:1']
+            }
+        }
+
+        self.assertItemsEqual(result['l3vpn']['import_rt'],
+                              expected['l3vpn']['import_rt'])
+        self.assertItemsEqual(result['l3vpn']['export_rt'],
+                              expected['l3vpn']['export_rt'])
+        self.assertItemsEqual(result['l2vpn']['import_rt'],
+                              expected['l2vpn']['import_rt'])
+        self.assertItemsEqual(result['l2vpn']['export_rt'],
+                              expected['l2vpn']['export_rt'])
+
+
+class TestBagpipeServiceDriverV2RPCs(TestBagpipeCommon,
+                                     TestBagpipeOVOPushPullMixin):
+    '''Check RPC push/pull and local registry callback effects'''
+
+    def setUp(self):
+        cfg.CONF.set_override('mechanism_drivers',
+                              ['logger', 'fake_agent'],
+                              'ml2')
+
+        super(TestBagpipeServiceDriverV2RPCs, self).setUp(
+            "%s.%s" % (__name__, TestCorePluginML2WithAgents.__name__),
+            driver=('networking_bgpvpn.neutron.services.service_drivers.'
+                    'bagpipe.bagpipe_v2.BaGPipeBGPVPNDriver'))
+
+    @mock.patch.object(resources_rpc.ResourcesPushRpcApi, 'push')
+    def test_router_itf_event_router_assoc(self, mocked_push):
+        with self.network() as net, \
+                self.subnet(network=net) as subnet, \
+                self.router(tenant_id=self._tenant_id) as router, \
+                self.bgpvpn() as bgpvpn, \
+                self.assoc_router(bgpvpn['bgpvpn']['id'],
+                                  router['router']['id']):
+
+            mocked_push.reset_mock()
+
+            itf = self._router_interface_action('add',
+                                                router['router']['id'],
+                                                subnet['subnet']['id'],
+                                                None)
+
+            mocked_push.assert_any_call(
+                mock.ANY,
+                [AnyOfClass(objs.BGPVPNRouterAssociation)], 'updated')
+
+            mocked_push.reset_mock()
+
+            itf = self._router_interface_action('remove',
+                                                router['router']['id'],
+                                                subnet['subnet']['id'],
+                                                itf['port_id'])
+            mocked_push.assert_any_call(
+                mock.ANY,
+                [AnyOfClass(objs.BGPVPNRouterAssociation)], 'updated')
+
+    @mock.patch.object(resources_rpc.ResourcesPushRpcApi, 'push')
+    def test_router_itf_event_network_assoc(self, mocked_push):
+        with self.network() as net, \
+                self.subnet(network=net) as subnet, \
+                self.router(tenant_id=self._tenant_id) as router, \
+                self.bgpvpn() as bgpvpn, \
+                self.assoc_net(bgpvpn['bgpvpn']['id'],
+                               net['network']['id']):
+
+            mocked_push.reset_mock()
+
+            itf = self._router_interface_action('add',
+                                                router['router']['id'],
+                                                subnet['subnet']['id'],
+                                                None)
+
+            mocked_push.assert_any_call(
+                mock.ANY,
+                [AnyOfClass(objs.BGPVPNNetAssociation)], 'updated')
+
+            mocked_push.reset_mock()
+
+            itf = self._router_interface_action('remove',
+                                                router['router']['id'],
+                                                subnet['subnet']['id'],
+                                                itf['port_id'])
+
+            mocked_push.assert_any_call(
+                mock.ANY,
+                [AnyOfClass(objs.BGPVPNNetAssociation)], 'updated')
+
+            ovos_in_call = mocked_push.mock_calls[0][1][1]
+            for ovo in ovos_in_call:
+                if not isinstance(ovo, objs.BGPVPNNetAssociation):
+                    continue
+                for subnet in ovo.all_subnets(net['network']['id']):
+                    self.assertIsNone(subnet['gateway_mac'])
