@@ -28,6 +28,7 @@ from neutron.tests.unit.db import test_db_base_plugin_v2
 from neutron.tests.unit.extensions import test_l3
 from neutron.tests.unit.extensions.test_l3 import TestL3NatServicePlugin
 from neutron_lib.api.definitions import bgpvpn as bgpvpn_def
+from neutron_lib.api.definitions import bgpvpn_vni as bgpvpn_vni_def
 
 from networking_bgpvpn.neutron.db import bgpvpn_db
 from networking_bgpvpn.neutron import extensions
@@ -45,6 +46,15 @@ def http_client_error(req, res):
                                      explanation=explanation)
 
 
+class TestBgpvpnDriverWithVni(driver_api.BGPVPNDriverRC):
+    more_supported_extension_aliases = (
+        driver_api.BGPVPNDriverRC.more_supported_extension_aliases +
+        [bgpvpn_vni_def.ALIAS])
+
+    def __init__(self, *args, **kwargs):
+        super(TestBgpvpnDriverWithVni, self).__init__(*args, **kwargs)
+
+
 class BgpvpnTestCaseMixin(test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
                           test_l3.L3NatTestCaseMixin):
 
@@ -52,7 +62,7 @@ class BgpvpnTestCaseMixin(test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
         if not service_provider:
             provider = (bgpvpn_def.LABEL +
                         ':dummy:networking_bgpvpn.neutron.services.'
-                        'service_drivers.driver_api.BGPVPNDriver:default')
+                        'service_drivers.driver_api.BGPVPNDriverRC:default')
         else:
             provider = (bgpvpn_def.LABEL + ':test:' + service_provider +
                         ':default')
@@ -106,6 +116,12 @@ class BgpvpnTestCaseMixin(test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
                                               'import_targets': [],
                                               'route_distinguishers': []})
 
+    def add_tenant(self, data):
+        data.update({
+            "project_id": self._tenant_id,
+            "tenant_id": self._tenant_id
+        })
+
     @contextlib.contextmanager
     def bgpvpn(self, do_delete=True, **kwargs):
         req_data = copy.deepcopy(self.bgpvpn_data)
@@ -180,6 +196,43 @@ class BgpvpnTestCaseMixin(test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
             if res.status_int >= 400:
                 raise http_client_error(del_req, res)
 
+    @contextlib.contextmanager
+    def assoc_port(self, bgpvpn_id, port_id, do_disassociate=True, **kwargs):
+        fmt = 'json'
+        data = {'port_association': {'port_id': port_id,
+                                     'tenant_id': self._tenant_id}}
+        data['port_association'].update(kwargs)
+        req = self.new_create_request(
+            'bgpvpn/bgpvpns',
+            data=data,
+            fmt=fmt,
+            id=bgpvpn_id,
+            subresource='port_associations')
+        res = req.get_response(self.ext_api)
+        if res.status_int >= 400:
+            raise http_client_error(req, res)
+        assoc = self.deserialize('json', res)
+        yield assoc
+        if do_disassociate:
+            del_req = self.new_delete_request(
+                'bgpvpn/bgpvpns',
+                bgpvpn_id,
+                fmt=self.fmt,
+                subresource='port_associations',
+                sub_id=assoc['port_association']['id'])
+            res = del_req.get_response(self.ext_api)
+            if res.status_int >= 400:
+                raise http_client_error(del_req, res)
+
+    def show_port_assoc(self, bgpvpn_id, port_assoc_id):
+        req = self.new_show_request("bgpvpn/bgpvpns", bgpvpn_id,
+                                    subresource="port_associations",
+                                    sub_id=port_assoc_id)
+        res = req.get_response(self.ext_api)
+        if res.status_int >= 400:
+            raise http_client_error(req, res)
+        return self.deserialize('json', res)
+
 
 class TestBGPVPNServicePlugin(BgpvpnTestCaseMixin):
 
@@ -266,6 +319,33 @@ class TestBGPVPNServicePlugin(BgpvpnTestCaseMixin):
             mock_validate.assert_called_once_with(
                 mock.ANY, router['router']['id'])
 
+    def test_bgpvpn_router_assoc_update(self):
+        with self.router(tenant_id=self._tenant_id) as router, \
+                self.bgpvpn() as bgpvpn, \
+                mock.patch.object(
+                    self.bgpvpn_plugin,
+                    '_validate_router',
+                    return_value=router['router']), \
+                self.assoc_router(bgpvpn['bgpvpn']['id'],
+                                  router['router']['id']) as router_assoc:
+
+            bgpvpn_id = bgpvpn['bgpvpn']['id']
+
+            updated = self._update('bgpvpn/bgpvpns/%s/router_associations' %
+                                   bgpvpn_id,
+                                   router_assoc['router_association']['id'],
+                                   {'router_association':
+                                       {'advertise_extra_routes': False}}
+                                   )
+            expected = {'router_association': {
+                'id': router_assoc['router_association']['id'],
+                'router_id': router['router']['id'],
+                'advertise_extra_routes': False
+            }}
+            self.add_tenant(expected['router_association'])
+
+            self.assertEqual(expected, updated)
+
     def test_associate_empty_router(self):
         with self.bgpvpn() as bgpvpn:
             id = bgpvpn['bgpvpn']['id']
@@ -477,6 +557,213 @@ class TestBGPVPNServicePlugin(BgpvpnTestCaseMixin):
             res = bgpvpn_rtr_intf_req.get_response(self.ext_api)
             self.assertEqual(res.status_int, webob.exc.HTTPConflict.code)
 
+    @mock.patch.object(plugin.BGPVPNPlugin,
+                       '_validate_port_association_routes_bgpvpn')
+    def test_bgpvpn_port_assoc_create(self, mock_validate_port_assoc):
+        with self.network() as net, \
+                self.subnet(network={'network': net['network']}) as subnet, \
+                self.port(subnet={'subnet': subnet['subnet']}) as port, \
+                self.bgpvpn() as bgpvpn, \
+                mock.patch.object(
+                    self.bgpvpn_plugin,
+                    '_validate_port',
+                    return_value=port['port']) as mock_validate, \
+                self.assoc_port(bgpvpn['bgpvpn']['id'],
+                                port['port']['id'],
+                                advertise_fixed_ips=False,
+                                routes=[{
+                                    'type': 'prefix',
+                                    'prefix': '12.1.3.0/24',
+                                    }]):
+            mock_validate.assert_called_once_with(
+                mock.ANY, port['port']['id'])
+
+            mock_validate_port_assoc.assert_called_once()
+
+    def _test_bgpvpn_port_assoc_create_incorrect(self, **kwargs):
+        with self.network() as net, \
+                self.subnet(network={'network': net['network']}) as subnet, \
+                self.port(subnet={'subnet': subnet['subnet']}) as port, \
+                self.bgpvpn() as bgpvpn:
+
+            data = {'port_association': {'port_id': port['port']['id'],
+                                         'tenant_id': self._tenant_id}}
+            data['port_association'].update(kwargs)
+            bgpvpn_net_req = self.new_create_request(
+                'bgpvpn/bgpvpns',
+                data=data,
+                fmt=self.fmt,
+                id=bgpvpn['bgpvpn']['id'],
+                subresource='port_associations')
+            res = bgpvpn_net_req.get_response(self.ext_api)
+            self.assertEqual(res.status_int, webob.exc.HTTPBadRequest.code)
+            return res.body
+
+    def test_bgpvpn_port_assoc_create_bgpvpn_route_non_existing(self):
+        with self.network() as net, \
+                self.subnet(network={'network': net['network']}) as subnet, \
+                self.port(subnet={'subnet': subnet['subnet']}) as port, \
+                self.bgpvpn() as bgpvpn:
+
+            data = {'port_association': {
+                    'port_id': port['port']['id'],
+                    'tenant_id': self._tenant_id,
+                    'routes': [{
+                        'type': 'bgpvpn',
+                        'bgpvpn_id': '3aff9b6b-387b-4ffd-a9ff-a4bdffb349ff'
+                        }]
+                    }}
+
+            bgpvpn_net_req = self.new_create_request(
+                'bgpvpn/bgpvpns',
+                data=data,
+                fmt=self.fmt,
+                id=bgpvpn['bgpvpn']['id'],
+                subresource='port_associations')
+            res = bgpvpn_net_req.get_response(self.ext_api)
+            self.assertEqual(res.status_int, webob.exc.HTTPBadRequest.code)
+            self.assertIn("bgpvpn specified in route does not exist",
+                          str(res.body))
+
+    def test_bgpvpn_port_assoc_create_bgpvpn_route_wrong_tenant(self):
+        with self.network() as net, \
+                self.subnet(network={'network': net['network']}) as subnet, \
+                self.port(subnet={'subnet': subnet['subnet']}) as port, \
+                self.bgpvpn() as bgpvpn, \
+                self.bgpvpn(tenant_id="notus") as bgpvpn_other:
+
+            data = {'port_association': {
+                    'port_id': port['port']['id'],
+                    'tenant_id': self._tenant_id,
+                    'routes': [{
+                        'type': 'bgpvpn',
+                        'bgpvpn_id': bgpvpn_other['bgpvpn']['id']
+                        }]
+                    }}
+
+            bgpvpn_net_req = self.new_create_request(
+                'bgpvpn/bgpvpns',
+                data=data,
+                fmt=self.fmt,
+                id=bgpvpn['bgpvpn']['id'],
+                subresource='port_associations')
+            res = bgpvpn_net_req.get_response(self.ext_api)
+            self.assertEqual(res.status_int, webob.exc.HTTPBadRequest.code)
+            self.assertIn("bgpvpn specified in route does not belong to "
+                          "the tenant", str(res.body))
+
+    def test_associate_empty_port(self):
+        with self.bgpvpn() as bgpvpn:
+            id = bgpvpn['bgpvpn']['id']
+            data = {}
+            bgpvpn_port_req = self.new_create_request(
+                'bgpvpn/bgpvpns',
+                data=data,
+                fmt=self.fmt,
+                id=id,
+                subresource='port_associations')
+            res = bgpvpn_port_req.get_response(self.ext_api)
+            self.assertEqual(res.status_int, webob.exc.HTTPBadRequest.code)
+            self.assertIn("Resource body required", str(res.body))
+
+    def test_associate_unknown_port(self):
+        with self.bgpvpn() as bgpvpn:
+            id = bgpvpn['bgpvpn']['id']
+            port_id = _uuid()
+            data = {'port_association': {'port_id': port_id,
+                                         'tenant_id': self._tenant_id}}
+            bgpvpn_port_req = self.new_create_request(
+                'bgpvpn/bgpvpns',
+                data=data,
+                fmt=self.fmt,
+                id=id,
+                subresource='port_associations')
+            res = bgpvpn_port_req.get_response(self.ext_api)
+            self.assertEqual(res.status_int, webob.exc.HTTPNotFound.code)
+
+    def test_associate_unauthorized_port(self):
+        with self.port(tenant_id=self._tenant_id) as port:
+            port_id = port['port']['id']
+            with self.bgpvpn(tenant_id='another_tenant') as bgpvpn:
+                id = bgpvpn['bgpvpn']['id']
+                data = {'port_association': {'port_id': port_id,
+                                             'tenant_id': self._tenant_id}}
+                bgpvpn_port_req = self.new_create_request(
+                    'bgpvpn/bgpvpns',
+                    data=data,
+                    fmt=self.fmt,
+                    id=id,
+                    subresource='port_associations')
+                res = bgpvpn_port_req.get_response(self.ext_api)
+                self.assertEqual(res.status_int, webob.exc.HTTPForbidden.code)
+
+    def test_port_assoc_belong_to_diff_tenant(self):
+        with self.port(tenant_id=self._tenant_id) as port:
+            port_id = port['port']['id']
+            with self.bgpvpn() as bgpvpn:
+                id = bgpvpn['bgpvpn']['id']
+                data = {'port_association': {'port_id': port_id,
+                                             'tenant_id': 'another_tenant'}}
+                bgpvpn_port_req = self.new_create_request(
+                    'bgpvpn/bgpvpns',
+                    data=data,
+                    fmt=self.fmt,
+                    id=id,
+                    subresource='port_associations')
+                res = bgpvpn_port_req.get_response(self.ext_api)
+                self.assertEqual(res.status_int, webob.exc.HTTPForbidden.code)
+
+    @mock.patch.object(plugin.BGPVPNPlugin,
+                       '_validate_port_association_routes_bgpvpn')
+    def test_bgpvpn_port_assoc_update(
+            self,
+            mock_validate):
+        with self.network() as net, \
+                self.subnet(network={'network': net['network']}) as subnet, \
+                self.port(subnet={'subnet': subnet['subnet']}) as port, \
+                self.bgpvpn() as bgpvpn, \
+                self.assoc_port(bgpvpn['bgpvpn']['id'],
+                                port['port']['id']) as port_assoc:
+            bgpvpn_id = bgpvpn['bgpvpn']['id']
+
+            self._update('bgpvpn/bgpvpns/%s/port_associations' % bgpvpn_id,
+                         port_assoc['port_association']['id'],
+                         {'port_association': {'advertise_fixed_ips': False}}
+                         )
+
+            # one call for create, one call for update
+            self.assertEqual(2, mock_validate.call_count)
+
+    def test_bgpvpn_port_assoc_update_bgpvpn_route_wrong_tenant(self):
+        with self.network() as net, \
+                self.subnet(network={'network': net['network']}) as subnet, \
+                self.port(subnet={'subnet': subnet['subnet']}) as port, \
+                self.bgpvpn() as bgpvpn, \
+                self.bgpvpn(tenant_id="not-us") as bgpvpn_other, \
+                self.assoc_port(bgpvpn['bgpvpn']['id'],
+                                port['port']['id']) as port_assoc:
+
+            bgpvpn_id = bgpvpn['bgpvpn']['id']
+
+            req = self.new_update_request(
+                'bgpvpn/bgpvpns/%s/port_associations' % bgpvpn_id,
+                {'port_association': {
+                    'routes': [{
+                        'type': 'bgpvpn',
+                        'bgpvpn_id': bgpvpn_other['bgpvpn']['id']
+                        }]
+                    }
+                 },
+                port_assoc['port_association']['id']
+            )
+
+            res = req.get_response(self.ext_api)
+
+            self.assertEqual(res.status_int, webob.exc.HTTPBadRequest.code)
+            self.assertIn(
+                "bgpvpn specified in route does not belong to the tenant",
+                str(res.body))
+
 
 class TestBGPVPNServiceDriverDB(BgpvpnTestCaseMixin):
 
@@ -541,21 +828,17 @@ class TestBGPVPNServiceDriverDB(BgpvpnTestCaseMixin):
     @mock.patch.object(driver_api.BGPVPNDriver,
                        'delete_bgpvpn_postcommit')
     def test_delete_bgpvpn(self, mock_delete_postcommit):
-        with self.bgpvpn(do_delete=False) as bgpvpn:
-            patcher = mock.patch.object(bgpvpn_db.BGPVPNPluginDb,
-                                        'delete_bgpvpn',
-                                        return_value=self.converted_data)
-            mock_delete_db = patcher.start()
-
+        with self.bgpvpn(do_delete=False) as bgpvpn, \
+                mock.patch.object(bgpvpn_db.BGPVPNPluginDb, 'delete_bgpvpn') \
+                as mock_delete_db, \
+                mock.patch.object(bgpvpn_db.BGPVPNPluginDb, 'get_bgpvpn',
+                                  return_value=self.converted_data):
             self._delete('bgpvpn/bgpvpns',
                          bgpvpn['bgpvpn']['id'])
             mock_delete_db.assert_called_once_with(mock.ANY,
                                                    bgpvpn['bgpvpn']['id'])
             mock_delete_postcommit.assert_called_once_with(mock.ANY,
                                                            self.converted_data)
-            patcher.stop()
-            self._delete('bgpvpn/bgpvpns',
-                         bgpvpn['bgpvpn']['id'])
 
     @mock.patch.object(bgpvpn_db.BGPVPNPluginDb, 'get_bgpvpn')
     def test_get_bgpvpn(self, mock_get_db):
@@ -598,9 +881,12 @@ class TestBGPVPNServiceDriverDB(BgpvpnTestCaseMixin):
             old_bgpvpn['id'] = bgpvpn['bgpvpn']['id']
             old_bgpvpn['networks'] = []
             old_bgpvpn['routers'] = []
+            old_bgpvpn['ports'] = []
             old_bgpvpn['project_id'] = old_bgpvpn['tenant_id']
+            old_bgpvpn['local_pref'] = None
             new_bgpvpn = copy.copy(old_bgpvpn)
-            new_bgpvpn['name'] = 'foo'
+            update = {'name': 'foo'}
+            new_bgpvpn.update(update)
 
             mock_update_db.return_value = new_bgpvpn
 
@@ -752,6 +1038,7 @@ class TestBGPVPNServiceDriverDB(BgpvpnTestCaseMixin):
                     net_assoc = {'id': assoc_id,
                                  'network_id': net_id,
                                  'bgpvpn_id': bgpvpn_id}
+                    self.add_tenant(net_assoc)
                     mock_db_del.return_value = net_assoc
             mock_db_del.assert_called_once_with(mock.ANY,
                                                 assoc_id,
@@ -879,10 +1166,201 @@ class TestBGPVPNServiceDriverDB(BgpvpnTestCaseMixin):
                     assoc_id = assoc['router_association']['id']
                     router_assoc = {'id': assoc_id,
                                     'router_id': router_id,
-                                    'bgpvpn_id': bgpvpn_id}
+                                    'bgpvpn_id': bgpvpn_id,
+                                    'advertise_extra_routes': True}
+                    self.add_tenant(router_assoc)
                     mock_db_del.return_value = router_assoc
+
+            # (delete triggered by exit from with statement)
+
             mock_db_del.assert_called_once_with(mock.ANY,
                                                 assoc_id,
                                                 bgpvpn_id)
             mock_postcommit.assert_called_once_with(mock.ANY,
                                                     router_assoc)
+
+    @mock.patch.object(driver_api.BGPVPNDriverRC,
+                       'create_port_assoc_postcommit')
+    @mock.patch.object(driver_api.BGPVPNDriverRC,
+                       'create_port_assoc_precommit')
+    @mock.patch.object(bgpvpn_db.BGPVPNPluginDb, 'create_port_assoc')
+    def test_create_bgpvpn_port_assoc(self, mock_db_create_assoc,
+                                      mock_pre_commit,
+                                      mock_post_commit):
+        with self.bgpvpn() as bgpvpn, \
+                self.network() as net,\
+                self.subnet(network={'network': net['network']}) as subnet,\
+                self.port(subnet={'subnet': subnet['subnet']},
+                          tenant_id=self._tenant_id) as port:
+            bgpvpn_id = bgpvpn['bgpvpn']['id']
+            port_id = port['port']['id']
+            assoc_id = _uuid()
+            data = {'tenant_id': self._tenant_id,
+                    'port_id': port_id}
+            port_assoc_dict = copy.copy(data)
+            port_assoc_dict.update({'id': assoc_id,
+                                    'bgpvpn_id': bgpvpn_id})
+            mock_db_create_assoc.return_value = port_assoc_dict
+            with self.assoc_port(bgpvpn_id, port_id=port_id,
+                                 do_disassociate=False):
+                self.assertTrue(mock_db_create_assoc.called)
+                self.assertEqual(
+                    bgpvpn_id, mock_db_create_assoc.call_args[0][1])
+                self.assertDictSupersetOf(
+                    data,
+                    mock_db_create_assoc.call_args[0][2])
+                mock_pre_commit.assert_called_once_with(mock.ANY,
+                                                        port_assoc_dict)
+                mock_post_commit.assert_called_once_with(mock.ANY,
+                                                         port_assoc_dict)
+
+    def test_create_bgpvpn_port_assoc_precommit_fails(self):
+        with self.bgpvpn() as bgpvpn, \
+                self.port(tenant_id=self._tenant_id) as port, \
+                mock.patch.object(driver_api.BGPVPNDriverRC,
+                                  'create_port_assoc_precommit',
+                                  new=self._raise_bgpvpn_driver_precommit_exc):
+            fmt = 'json'
+            data = {'port_association': {'port_id': port['port']['id'],
+                                         'tenant_id': self._tenant_id}}
+            bgpvpn_port_req = self.new_create_request(
+                'bgpvpn/bgpvpns',
+                data=data,
+                fmt=fmt,
+                id=bgpvpn['bgpvpn']['id'],
+                subresource='port_associations')
+            res = bgpvpn_port_req.get_response(self.ext_api)
+            # Assert that driver failure returns an error
+            self.assertEqual(webob.exc.HTTPError.code,
+                             res.status_int)
+            # Assert that the bgpvpn is not associated to network
+            bgpvpn_new = self._show('bgpvpn/bgpvpns',
+                                    bgpvpn['bgpvpn']['id'])
+            self.assertEqual([], bgpvpn_new['bgpvpn']['ports'])
+
+    @mock.patch.object(bgpvpn_db.BGPVPNPluginDb, 'get_port_assoc')
+    def test_get_bgpvpn_port_assoc(self, mock_get_db):
+        with self.bgpvpn() as bgpvpn, \
+                self.port(tenant_id=self._tenant_id) as port, \
+                self.assoc_port(bgpvpn['bgpvpn']['id'],
+                                port['port']['id']) as assoc:
+            bgpvpn_id = bgpvpn['bgpvpn']['id']
+            assoc_id = assoc['port_association']['id']
+            res = 'bgpvpn/bgpvpns/' + bgpvpn_id + \
+                  '/port_associations'
+            self._show(res, assoc_id)
+            mock_get_db.assert_called_once_with(mock.ANY,
+                                                assoc_id,
+                                                bgpvpn_id,
+                                                [])
+
+    @mock.patch.object(bgpvpn_db.BGPVPNPluginDb, 'get_port_assocs')
+    def test_get_bgpvpn_port_assoc_list(self, mock_get_db):
+        with self.bgpvpn() as bgpvpn, \
+                self.port(tenant_id=self._tenant_id) as port, \
+                self.assoc_port(bgpvpn['bgpvpn']['id'],
+                                port['port']['id']):
+            bgpvpn_id = bgpvpn['bgpvpn']['id']
+            res = 'bgpvpn/bgpvpns/' + bgpvpn_id + \
+                  '/port_associations'
+            self._list(res)
+            mock_get_db.assert_called_once_with(mock.ANY,
+                                                bgpvpn_id,
+                                                mock.ANY, mock.ANY)
+
+    @mock.patch.object(driver_api.BGPVPNDriverRC,
+                       'delete_port_assoc_precommit')
+    @mock.patch.object(bgpvpn_db.BGPVPNPluginDb, 'delete_port_assoc')
+    def test_delete_bgpvpn_port_assoc_precommit_fails(self, mock_db_del,
+                                                      mock_precommit):
+        with self.bgpvpn() as bgpvpn, \
+                self.port(tenant_id=self._tenant_id) as port, \
+                self.assoc_port(bgpvpn['bgpvpn']['id'],
+                                port['port']['id']) as assoc:
+            port_id = port['port']['id']
+            bgpvpn_id = bgpvpn['bgpvpn']['id']
+            assoc_id = assoc['port_association']['id']
+            port_assoc = {'id': assoc_id,
+                          'port_id': port_id,
+                          'bgpvpn_id': bgpvpn_id}
+            mock_db_del.return_value = port_assoc
+            mock_precommit.return_value = \
+                self._raise_bgpvpn_driver_precommit_exc
+            # Assert that existing bgpvpn and port-assoc remains
+            list = self._list('bgpvpn/bgpvpns', fmt='json')
+            bgpvpn['bgpvpn']['ports'] = [port_assoc['port_id']]
+            self.assertEqual([bgpvpn['bgpvpn']], list['bgpvpns'])
+
+    @mock.patch.object(driver_api.BGPVPNDriverRC,
+                       'update_port_assoc_precommit')
+    @mock.patch.object(driver_api.BGPVPNDriverRC,
+                       'update_port_assoc_postcommit')
+    @mock.patch.object(bgpvpn_db.BGPVPNPluginDb, 'update_port_assoc')
+    def test_update_bgpvpn_port_assoc(self, mock_db_update,
+                                      mock_postcommit, mock_precommit):
+        with self.bgpvpn() as bgpvpn, \
+                self.port(tenant_id=self._tenant_id) as port, \
+                self.assoc_port(bgpvpn['bgpvpn']['id'],
+                                port['port']['id']) as assoc:
+
+            bgpvpn_id = bgpvpn['bgpvpn']['id']
+            assoc_id = assoc['port_association']['id']
+
+            assoc['port_association'].update({'bgpvpn_id': bgpvpn_id})
+
+            new_port_assoc = copy.deepcopy(assoc)
+            changed = {'advertise_fixed_ips': False}
+
+            new_port_assoc['port_association'].update(changed)
+            mock_db_update.return_value = new_port_assoc['port_association']
+
+            data = {"port_association": changed}
+            self._update('bgpvpn/bgpvpns/%s/port_associations' % bgpvpn_id,
+                         assoc['port_association']['id'],
+                         data)
+
+            mock_db_update.assert_called_once_with(mock.ANY,
+                                                   assoc_id,
+                                                   bgpvpn_id,
+                                                   data['port_association'])
+            mock_precommit.assert_called_once_with(
+                mock.ANY,
+                assoc['port_association'],
+                new_port_assoc['port_association']
+                )
+            mock_postcommit.assert_called_once_with(
+                mock.ANY,
+                assoc['port_association'],
+                new_port_assoc['port_association']
+                )
+
+    @mock.patch.object(driver_api.BGPVPNDriverRC,
+                       'delete_port_assoc_precommit')
+    @mock.patch.object(driver_api.BGPVPNDriverRC,
+                       'delete_port_assoc_postcommit')
+    @mock.patch.object(bgpvpn_db.BGPVPNPluginDb, 'delete_port_assoc')
+    def test_delete_bgpvpn_port_assoc(self, mock_db_del,
+                                      mock_postcommit, mock_precommit):
+        with self.bgpvpn() as bgpvpn, \
+                self.port(tenant_id=self._tenant_id) as port, \
+                self.assoc_port(bgpvpn['bgpvpn']['id'],
+                                port['port']['id']) as assoc:
+            port_id = port['port']['id']
+            bgpvpn_id = bgpvpn['bgpvpn']['id']
+            assoc_id = assoc['port_association']['id']
+
+            port_assoc = {'id': assoc_id,
+                          'bgpvpn_id': bgpvpn_id,
+                          'port_id': port_id,
+                          'routes': [],
+                          'advertise_fixed_ips': True}
+            self.add_tenant(port_assoc)
+            mock_db_del.return_value = port_assoc
+
+        # (delete triggered by exit from with statement)
+
+        mock_db_del.assert_called_once_with(mock.ANY,
+                                            assoc_id,
+                                            bgpvpn_id)
+        mock_precommit.assert_called_once_with(mock.ANY, port_assoc)
+        mock_postcommit.assert_called_once_with(mock.ANY, port_assoc)
