@@ -15,6 +15,7 @@
 
 from networking_bgpvpn_tempest.services.bgpvpn import bgpvpn_client
 import tempest.api.network.base as test
+from tempest.common import utils
 from tempest import config
 from tempest.lib.common.utils import data_utils
 
@@ -24,11 +25,6 @@ CONF = config.CONF
 class BaseBgpvpnTest(test.BaseNetworkTest):
     """Base class for the Bgpvpn tests that use the Tempest Neutron REST client
 
-    Finally, it is assumed that the following option is defined in the
-    [service_available] section of etc/tempest.conf
-
-        bgpvpn as True
-
     """
 
     credentials = ['primary', 'admin', 'alt']
@@ -47,36 +43,37 @@ class BaseBgpvpnTest(test.BaseNetworkTest):
         cls.route_distinguishers = []
         cls.bgpvpns = []
         cls.bgpvpn_client = bgpvpn_client.BgpvpnClient(
-            cls.os.auth_provider,
+            cls.os_primary.auth_provider,
             CONF.network.catalog_type,
             CONF.network.region or CONF.identity.region,
             endpoint_type=CONF.network.endpoint_type,
             build_interval=CONF.network.build_interval,
             build_timeout=CONF.network.build_timeout,
-            **cls.os.default_params)
+            **cls.os_primary.default_params)
         cls.bgpvpn_admin_client = bgpvpn_client.BgpvpnClient(
-            cls.os_adm.auth_provider,
+            cls.os_admin.auth_provider,
             CONF.network.catalog_type,
             CONF.network.region or CONF.identity.region,
             endpoint_type=CONF.network.endpoint_type,
             build_interval=CONF.network.build_interval,
             build_timeout=CONF.network.build_timeout,
-            **cls.os_adm.default_params)
+            **cls.os_admin.default_params)
         cls.bgpvpn_alt_client = bgpvpn_client.BgpvpnClient(
-            cls.alt_manager.auth_provider,
+            cls.os_alt.auth_provider,
             CONF.network.catalog_type,
             CONF.network.region or CONF.identity.region,
             endpoint_type=CONF.network.endpoint_type,
             build_interval=CONF.network.build_interval,
             build_timeout=CONF.network.build_timeout,
-            **cls.alt_manager.default_params)
+            **cls.os_alt.default_params)
         super(BaseBgpvpnTest, cls).resource_setup()
 
     @classmethod
     def skip_checks(cls):
         super(BaseBgpvpnTest, cls).skip_checks()
-        if not CONF.service_available.bgpvpn:
-            raise cls.skipException("Bgpvpn support is required")
+        if not utils.is_extension_enabled('bgpvpn', 'network'):
+            msg = "Bgpvpn extension not enabled."
+            raise cls.skipException(msg)
 
     def create_bgpvpn(self, client, **kwargs):
         if 'name' not in kwargs:
@@ -86,3 +83,7 @@ class BaseBgpvpnTest(test.BaseNetworkTest):
         bgpvpn = body['bgpvpn']
         self.bgpvpns.append(bgpvpn)
         return bgpvpn
+
+    def delete_bgpvpn(self, client, bgpvpn):
+        client.delete_bgpvpn(bgpvpn['id'])
+        self.bgpvpns.remove(bgpvpn)
